#include "scene.h"

#include <iostream>
#include <fstream>

#include <QFileInfo>
#include <QTextStream>
#include <QInputDialog>
#include <QtCore/qglobal.h>

#include <QGLViewer/qglviewer.h>

#include "function_sphere.h"
#include "function_torus.h"

Scene::Scene()
{
	// view options
	m_view_mesh = true;
	m_view_edges = true;
	m_view_vertices = true;
}

Scene::~Scene()
{
}

void Scene::render()
{
	::glDisable(GL_LIGHTING);

	if(m_view_vertices) 
		m_c3t3.render_vertices(3.0f, 0, 0, 0);

	if(m_view_edges)
		m_c3t3.render_edges(1.0f, 128, 128, 128);

	if(m_view_mesh)
		m_c3t3.render_cells(0.0);
}

void Scene::mesh_sphere(const FT angle, 
	const FT sizing, 
	const FT approximation)
{
	// Define domain (careful: Sphere_3 constructor uses squared radius)
	Sphere bounding_sphere(CGAL::ORIGIN, 1.5 * 1.5);
	Mesh_domain domain(function_sphere, bounding_sphere); // unit sphere

	// set mesh criteria
	Mesh_criteria criteria(facet_angle = angle, 
		facet_size = sizing, 
		facet_distance = approximation,
		cell_radius_edge = 2.0, 
		cell_size = sizing);

	// Mesh generation
	m_c3t3.clear();
	std::cout << "Meshing a unit sphere...";
	m_c3t3 = CGAL::make_mesh_3<C3t3>(domain, criteria);
	std::cout << "done" << std::endl;
}

void Scene::mesh_torus(const FT angle, 
	const FT sizing, 
	const FT approximation)
{
}

void Scene::mesh_ellipsoid(const FT angle, 
	const FT sizing, 
	const FT approximation)
{
}



